////////////////////////////////////////////////////////////////////////
//
// ExternalOnlyFilter class:
// Algoritm to produce a filtered event file having
// neutrino vertex exclusively outside of the TPC
//
// corey.adams@yale.edu
//
////////////////////////////////////////////////////////////////////////
#ifndef BASEFILTER_H
#define BASEFILTER_H


#include "TH1D.h"
#include <string>

/// Framework includes
#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art_root_io/TFileService.h"
#include "art_root_io/TFileDirectory.h"
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "canvas/Persistency/Common/FindMany.h"

#include "nusimdata/SimulationBase/MCTruth.h"

#include "TVector3.h"

namespace filt {

class ExternalOnlyFilter : public art::EDFilter  {

public:

  explicit ExternalOnlyFilter(fhicl::ParameterSet const& pset);
  virtual ~ExternalOnlyFilter();


  bool filter(art::Event& evt);
  void reconfigure(fhicl::ParameterSet const& p);
  void beginJob();


private:

  bool IsFiducial(const TVector3 & vtx);


  std::string fgenerator_label;

protected:


}; // class ExternalOnlyFilter

//-------------------------------------------------
ExternalOnlyFilter::ExternalOnlyFilter(fhicl::ParameterSet const & pset)
: EDFilter(pset)
{
  this->reconfigure(pset);
}

//-------------------------------------------------
ExternalOnlyFilter::~ExternalOnlyFilter()
{
}

//-------------------------------------------------
void ExternalOnlyFilter::reconfigure(fhicl::ParameterSet const& p)
{
  fgenerator_label = p.get< std::string >("GeneratorLabel");
}

//-------------------------------------------------
void ExternalOnlyFilter::beginJob()
{
}

//-------------------------------------------------
bool ExternalOnlyFilter::filter(art::Event &evt)
{

  art::Handle< std::vector<simb::MCTruth> > truthHandle;
  evt.getByLabel(fgenerator_label, truthHandle);

  // Get the neutrino vertex location:
  auto vertex = truthHandle -> front().GetNeutrino().Lepton().Position().Vect();

  if (IsFiducial(vertex)) {
    return false;
  }
  else {
    return true;
  }

} // bool

bool ExternalOnlyFilter::IsFiducial(const TVector3 & vtx) {

  // Check against the 3 coordinates:
  if (vtx.X() > 23.75 || vtx.X() < -23.75 )
  {
    return false;
  }
  if (vtx.Y() > 20.0 || vtx.Y() < -20.0 )
  {
    return false;
  }
  if (vtx.Z() > 90.0 || vtx.Z() < 0.0)
  {
    return false;
  }
  return true;

}


DEFINE_ART_MODULE(ExternalOnlyFilter)

} //namespace filt

#endif // MATCHFILTER_H
